@@ -1,7 +1,7 @@
-/* $Id: control.cc 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -209,11 +209,16 @@ bool performNetPerfMeterAddFlow(MessageReader* messageReader,
    strncpy((char*)&addFlowMsg->CongestionControl, flow->getTrafficSpec().CongestionControl.c_str(),
            std::min(sizeof(addFlowMsg->CongestionControl), flow->getTrafficSpec().CongestionControl.size()));
 
-   addFlowMsg->NDiffPorts = htons(flow->getTrafficSpec().NDiffPorts);
    memset((char*)&addFlowMsg->PathMgr, 0, sizeof(addFlowMsg->PathMgr));
    strncpy((char*)&addFlowMsg->PathMgr, flow->getTrafficSpec().PathMgr.c_str(),
            std::min(sizeof(addFlowMsg->PathMgr), flow->getTrafficSpec().PathMgr.size()));
 
+   memset((char*)&addFlowMsg->Scheduler, 0, sizeof(addFlowMsg->Scheduler));
+   strncpy((char*)&addFlowMsg->Scheduler, flow->getTrafficSpec().Scheduler.c_str(),
+           std::min(sizeof(addFlowMsg->Scheduler), flow->getTrafficSpec().Scheduler.size()));
+
+   addFlowMsg->NDiffPorts = htons(flow->getTrafficSpec().NDiffPorts);
+
    sctp_sndrcvinfo sinfo;
    memset(&sinfo, 0, sizeof(sinfo));
    sinfo.sinfo_ppid = htonl(PPID_NETPERFMETER_CONTROL);
@@ -370,6 +375,7 @@ bool performNetPerfMeterStart(MessageReader*         messageReader,
          fprintf(configFile, "FLOW%u_DEBUG=\"%s\"\n",                        flow->getFlowID(), (flow->getTrafficSpec().Debug == true) ? "on" : "off");
          fprintf(configFile, "FLOW%u_CC=\"%s\"\n",                           flow->getFlowID(), flow->getTrafficSpec().CongestionControl.c_str());
          fprintf(configFile, "FLOW%u_PATHMGR=\"%s\"\n",                      flow->getFlowID(), flow->getTrafficSpec().PathMgr.c_str());
+         fprintf(configFile, "FLOW%u_SCHEDULER=\"%s\"\n",                    flow->getFlowID(), flow->getTrafficSpec().Scheduler.c_str());
          fprintf(configFile, "FLOW%u_NDIFFPORTS=%u\n",                       flow->getFlowID(), flow->getTrafficSpec().NDiffPorts);
          fprintf(configFile, "FLOW%u_VECTOR_ACTIVE_NODE=\"%s\"\n",           flow->getFlowID(), flow->getVectorFile().getName().c_str());
          fprintf(configFile, "FLOW%u_VECTOR_PASSIVE_NODE=\"%s\"\n\n",        flow->getFlowID(),
@@ -800,7 +806,13 @@ static bool handleNetPerfMeterAddFlow(MessageReader*                    messageR
       char pathMgr[sizeof(addFlowMsg->PathMgr) + 1];
       memcpy((char*)&pathMgr, (const char*)&addFlowMsg->PathMgr, sizeof(addFlowMsg->PathMgr));
       pathMgr[sizeof(addFlowMsg->PathMgr)] = 0x00;
-      trafficSpec.PathMgr    = std::string(pathMgr);
+      trafficSpec.PathMgr = std::string(pathMgr);
+
+      char scheduler[sizeof(addFlowMsg->Scheduler) + 1];
+      memcpy((char*)&scheduler, (const char*)&addFlowMsg->Scheduler, sizeof(addFlowMsg->Scheduler));
+      scheduler[sizeof(addFlowMsg->Scheduler)] = 0x00;
+      trafficSpec.Scheduler = std::string(scheduler);
+
       trafficSpec.NDiffPorts = ntohs(addFlowMsg->NDiffPorts);
 
       Flow* flow = new Flow(ntoh64(addFlowMsg->MeasurementID), ntohl(addFlowMsg->FlowID),
@@ -1037,28 +1049,23 @@ bool handleNetPerfMeterControlMessage(MessageReader* messageReader,
             return(handleNetPerfMeterAddFlow(
                       messageReader, controlSocket,
                       (const NetPerfMeterAddFlowMessage*)&inputBuffer, received));
-          break;
          case NETPERFMETER_REMOVE_FLOW:
             return(handleNetPerfMeterRemoveFlow(
                       messageReader, controlSocket,
                       (const NetPerfMeterRemoveFlowMessage*)&inputBuffer, received));
-          break;
          case NETPERFMETER_START:
             return(handleNetPerfMeterStart(
                       messageReader, controlSocket,
                       (const NetPerfMeterStartMessage*)&inputBuffer, received));
-          break;
          case NETPERFMETER_STOP:
             return(handleNetPerfMeterStop(
                       messageReader, controlSocket,
                       (const NetPerfMeterStopMessage*)&inputBuffer, received));
-          break;
          default:
             std::cerr << "ERROR: Received invalid control message of type "
                      << (unsigned int)header->Type << "!" << std::endl;
             sendAbort(controlSocket);
             return(false);
-          break;
       }
    }
    return(true);
