@@ -1,7 +1,7 @@
-// $Id: createsummary.cc 1930 2014-10-01 15:49:11Z dreibh $
+// $Id$
 // ###########################################################################
 //             Thomas Dreibholz's R Simulation Scripts Collection
-//                  Copyright (C) 2004-2013 Thomas Dreibholz
+//                  Copyright (C) 2004-2016 Thomas Dreibholz
 //
 //           Author: Thomas Dreibholz, dreibh@iem.uni-due.de
 // ###########################################################################
@@ -108,8 +108,8 @@ ScalarNode::~ScalarNode()
 // ###### Get ScalarNode object from storage node ###########################
 static inline ScalarNode* getScalarNodeFromStorageNode(struct SimpleRedBlackTreeNode* node)
 {
-   const long offset = (long)(&((struct ScalarNode*)node)->Node) - (long)node;
-   return( (struct ScalarNode*)( (long)node - offset ));
+   const long offset = (long)(&((class ScalarNode*)node)->Node) - (long)node;
+   return( (class ScalarNode*)( (long)node - offset ));
 }
 
 
