@@ -1,8 +1,8 @@
-/* $Id: flow.cc 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -1219,26 +1219,15 @@ void Flow::run()
 // ###### Configure socket parameters #######################################
 bool Flow::configureSocket(const int socketDescriptor)
 {
-   if(TrafficSpec.RcvBufferSize != 0) {
-      int bufferSize = TrafficSpec.RcvBufferSize;
-      if(ext_setsockopt(socketDescriptor, SOL_SOCKET, SO_RCVBUF, &bufferSize, sizeof(bufferSize)) < 0) {
-         std::cerr << "ERROR: Failed to configure receive buffer size on SCTP socket (SO_RCVBUF option) - "
-                  << strerror(errno) << "!" << std::endl;
-         return(false);
-      }
-   }
-   if(TrafficSpec.SndBufferSize != 0) {
-      int bufferSize = TrafficSpec.SndBufferSize;
-      if(ext_setsockopt(socketDescriptor, SOL_SOCKET, SO_SNDBUF, &bufferSize, sizeof(bufferSize)) < 0) {
-         std::cerr << "ERROR: Failed to configure send buffer size on SCTP socket (SO_SNDBUF option) - "
-                  << strerror(errno) << "!" << std::endl;
-         return(false);
-      }
+   if(setBufferSizes(socketDescriptor,
+                     (int)TrafficSpec.SndBufferSize,
+                     (int)TrafficSpec.RcvBufferSize) == false) {
+      return(false);
    }
    if( (TrafficSpec.Protocol == IPPROTO_TCP) || (TrafficSpec.Protocol == IPPROTO_MPTCP) ) {
       const int noDelayOption = (TrafficSpec.NoDelay == true) ? 1 : 0;
       if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_NODELAY, (const char*)&noDelayOption, sizeof(noDelayOption)) < 0) {
-         std::cerr << "ERROR: Failed to set TCP_NODELAY on socket - "
+         std::cerr << "ERROR: Failed to set TCP_NODELAY - "
                << strerror(errno) << "!" << std::endl;
          return(false);
       }
@@ -1251,19 +1240,25 @@ bool Flow::configureSocket(const int socketDescriptor)
 
          const int debugOption = (TrafficSpec.Debug == true) ? 1 : 0;
          if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_DEBUG, (const char*)&debugOption, sizeof(debugOption)) < 0) {
-            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_DEBUG on socket - "
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_DEBUG on MPTCP socket - "
                       << strerror(errno) << "!" << std::endl;
          }
 
          const int nDiffPorts = (int)TrafficSpec.NDiffPorts;
          if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_NDIFFPORTS, (const char*)&nDiffPorts, sizeof(nDiffPorts)) < 0) {
-            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_NDIFFPORTS on socket - "
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_NDIFFPORTS on MPTCP socket - "
                       << strerror(errno) << "!" << std::endl;
          }
 
          const char* pathMgr = TrafficSpec.PathMgr.c_str();
          if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_PATHMANAGER, pathMgr, strlen(pathMgr)) < 0) {
-            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_PATHMANAGER on socket - "
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_PATHMANAGER on MPTCP socket - "
+                      << strerror(errno) << "!" << std::endl;
+         }
+
+         const char* scheduler = TrafficSpec.Scheduler.c_str();
+         if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_MULTIPATH_SCHEDULER, scheduler, strlen(scheduler)) < 0) {
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_SCHEDULER on MPTCP socket - "
                       << strerror(errno) << "!" << std::endl;
          }
 #endif
@@ -1275,7 +1270,7 @@ bool Flow::configureSocket(const int socketDescriptor)
       if(strcmp(congestionControl, "default") != 0) {
          if (ext_setsockopt(socketDescriptor, IPPROTO_TCP, TCP_CONGESTION, congestionControl, strlen(congestionControl)) < 0) {
             std::cerr << "ERROR: Failed to set TCP_CONGESTION ("
-                      << congestionControl << ") on socket - "
+                      << congestionControl << ") - "
                       << strerror(errno) << "!" << std::endl;
             return(false);
          }
@@ -1286,7 +1281,7 @@ bool Flow::configureSocket(const int socketDescriptor)
       if (TrafficSpec.NoDelay) {
          const int noDelayOption = 1;
          if (ext_setsockopt(socketDescriptor, IPPROTO_SCTP, SCTP_NODELAY, (const char*)&noDelayOption, sizeof(noDelayOption)) < 0) {
-            std::cerr << "ERROR: Failed to set SCTP_NODELAY on socket - "
+            std::cerr << "ERROR: Failed to set SCTP_NODELAY on SCTP socket - "
                   << strerror(errno) << "!" << std::endl;
             return(false);
          }
