@@ -1,8 +1,8 @@
-/* $Id: flowtrafficspec.cc 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -135,7 +135,8 @@ void FlowTrafficSpec::print(std::ostream& os) const
       << ((NoDelay == true) ? "yes" : "no") << std::endl
       << "      Congestion Control:    " << CongestionControl << std::endl
       << "      Number of Diff. Ports: " << NDiffPorts        << std::endl
-      << "      Path Manager:          " << PathMgr           << std::endl;
+      << "      Path Manager:          " << PathMgr           << std::endl
+      << "      Scheduler:             " << Scheduler         << std::endl;
 
    os << "      - CMT:                 #" << (unsigned int)CMT << " ";
    switch(CMT) {
@@ -174,9 +175,11 @@ void FlowTrafficSpec::reset()
    ErrorOnAbort             = true;
    Debug                    = false;
    NoDelay                  = false;
+   BindV6Only               = false;
    RepeatOnOff              = false;
    NDiffPorts               = 4;
-   PathMgr                  = "default";
+   PathMgr                  = "fullmesh";
+   Scheduler                = "default";
    CongestionControl        = "default";
    CMT                      = 0x00;
    CCID                     = 0x00;
