@@ -1,8 +1,8 @@
-/* $Id: flowtrafficspec.h 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -83,13 +83,15 @@ class FlowTrafficSpec
    uint8_t                 CCID;
 
    uint16_t                NDiffPorts;
-   std::string             PathMgr;
    std::string             CongestionControl;
+   std::string             PathMgr;
+   std::string             Scheduler;
 
    bool                    Debug;
    bool                    NoDelay;
    bool                    ErrorOnAbort;
    bool                    RepeatOnOff;
+   bool                    BindV6Only;
 
    std::vector<OnOffEvent> OnOffEvents;
 };
