@@ -1,8 +1,8 @@
-/* $Id: netperfmeter.cc 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
  * Copyright (C) 2013 by Sebastian Wallat (TCP No delay)
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,6 +46,11 @@ static sockaddr_union gLocalAddressArray[MAX_LOCAL_ADDRESSES];
 
 static const char*    gActiveNodeName   = "Client";
 static const char*    gPassiveNodeName  = "Server";
+static const char*    gPathMgr          = "fullmesh";
+static const char*    gScheduler        = "default";
+static bool           gBindV6Only       = false;
+static int            gSndBufSize       = -1;
+static int            gRcvBufSize       = -1;
 static bool           gControlOverTCP   = false;
 static int            gControlSocket    = -1;
 static int            gControlSocketTCP = -1;
@@ -75,6 +80,21 @@ bool handleGlobalParameter(char* parameter)
    else if(strncmp(parameter, "-passivenodename=", 17) == 0) {
       gPassiveNodeName = (const char*)&parameter[17];
    }
+   else if(strncmp(parameter, "-pathmgr=", 9) == 0) {
+      gPathMgr = (const char*)&parameter[9];
+   }
+   else if(strncmp(parameter, "-scheduler=", 11) == 0) {
+      gScheduler = (const char*)&parameter[11];
+   }
+   else if(strncmp(parameter, "-sndbuf=", 8) == 0) {
+      gSndBufSize = atol((const char*)&parameter[8]);
+   }
+   else if(strncmp(parameter, "-rcvbuf=", 8) == 0) {
+      gRcvBufSize = atol((const char*)&parameter[8]);
+   }
+   else if(strcmp(parameter, "-v6only") == 0) {
+      gBindV6Only = true;
+   }
    else if(strcmp(parameter, "-quiet") == 0) {
       // Already handled before!
    }
@@ -334,6 +354,10 @@ static const char* parseTrafficSpecOption(const char*      parameters,
          exit(1);
       }
    }
+   else if(strncmp(parameters, "v6only", 6) == 0) {
+      trafficSpec.BindV6Only = true;
+      n = 6;
+   }
    else if(sscanf(parameters, "description=%255[^:]s%n", (char*)&description, &n) == 1) {
       trafficSpec.Description = std::string(description);
       n = 12 + strlen(description);
@@ -440,6 +464,26 @@ static const char* parseTrafficSpecOption(const char*      parameters,
       trafficSpec.PathMgr = std::string((const char*)&pathMgr);
       n = 8 + strlen((const char*)&pathMgr);
    }
+   else if(strncmp(parameters, "scheduler=", 10) == 0) {
+      char   scheduler[NETPERFMETER_SCHEDULER_LENGTH + 1];
+      size_t i = 0;
+      while(i < NETPERFMETER_SCHEDULER_LENGTH) {
+         if( (parameters[10 + i] == ':') ||
+             (parameters[10 + i] == 0x00) ) {
+            break;
+         }
+         scheduler[i] = parameters[10 + i];
+         i++;
+      }
+      scheduler[i] = 0x00;
+      if( (parameters[10 + i] != ':') && (parameters[10 + i] != 0x00) ) {
+         cerr << "ERROR: Invalid \"scheduler\" setting: " << (const char*)&parameters[10]
+              << " - name too long!" << std::endl;
+          exit(1);
+      }
+      trafficSpec.Scheduler = std::string((const char*)&scheduler);
+      n = 10 + strlen((const char*)&scheduler);
+   }
    else if(strncmp(parameters, "cc=", 3) == 0) {
       char   congestionControl[NETPERFMETER_CC_LENGTH + 1];
       size_t i = 0;
@@ -568,31 +612,36 @@ static Flow* createFlow(Flow*                  previousFlow,
       switch(trafficSpec.Protocol) {
          case IPPROTO_SCTP:
             socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_STREAM, IPPROTO_SCTP, 0,
-                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
+                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false,
+                                                   trafficSpec.BindV6Only);
            break;
          case IPPROTO_TCP:
             socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_STREAM, IPPROTO_TCP, 0,
-                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
+                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false,
+                                                   trafficSpec.BindV6Only);
            break;
 #ifdef HAVE_MPTCP
          case IPPROTO_MPTCP:
             socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_STREAM, IPPROTO_MPTCP, 0,
-                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
+                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false,
+                                                   trafficSpec.BindV6Only);
            break;
 #endif
          case IPPROTO_UDP:
             socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_DGRAM, IPPROTO_UDP, 0,
-                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
+                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false,
+                                                   trafficSpec.BindV6Only);
            break;
 #ifdef HAVE_DCCP
          case IPPROTO_DCCP:
             socketDescriptor = createAndBindSocket(remoteAddress.sa.sa_family, SOCK_DCCP, IPPROTO_DCCP, 0,
-                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false);
+                                                   gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, false,
+                                                   trafficSpec.BindV6Only);
            break;
 #endif
          default:
             abort();
-          break;
+
       }
       if(socketDescriptor < 0) {
          cerr << "ERROR: Unable to create " << getProtocolName(trafficSpec.Protocol)
@@ -811,7 +860,7 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
 
    // ====== Initialize control socket ======================================
    gControlSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_SCTP,
-                                        localPort + 1, 0, NULL, true);
+                                        localPort + 1, 0, NULL, true, gBindV6Only);
    if(gControlSocket < 0) {
       cerr << "ERROR: Failed to create and bind SCTP socket for control port - "
            << strerror(errno) << "!" << endl;
@@ -831,7 +880,7 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
    gMessageReader.registerSocket(IPPROTO_SCTP, gControlSocket);
 
    gControlSocketTCP = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP,
-                                           localPort + 1, 0, NULL, true);
+                                           localPort + 1, 0, NULL, true, gBindV6Only);
    if(gControlSocketTCP < 0) {
       cerr << "ERROR: Failed to create and bind TCP socket for control port - "
            << strerror(errno) << "!" << endl;
@@ -841,26 +890,46 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
 
    // ====== Initialize data socket for each protocol =======================
    gTCPSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_TCP, localPort,
-                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);                                    
+                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true, gBindV6Only);
    if(gTCPSocket < 0) {
       cerr << "ERROR: Failed to create and bind TCP socket - "
            << strerror(errno) << "!" << endl;
       exit(1);
    }
+   if(setBufferSizes(gTCPSocket, gSndBufSize, gRcvBufSize) == false) {
+      exit(1);
+   }
 
 #ifdef HAVE_MPTCP
    gMPTCPSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_MPTCP, localPort - 1,
-                                      gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
+                                      gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true, gBindV6Only);
    if(gMPTCPSocket < 0) {
       if(gOutputVerbosity >= NPFOV_STATUS) {
          cerr << "NOTE: Unable to create and bind MPTCP socket - "
-            << strerror(errno) << "!" << endl;
+              << strerror(errno) << "!" << endl;
+      }
+   }
+   else {
+      if (ext_setsockopt(gMPTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_PATHMANAGER, gPathMgr, strlen(gPathMgr)) < 0) {
+         if(strcmp(gPathMgr, "default") != 0) {
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_PATHMANAGER on socket - "
+                        << strerror(errno) << "!" << std::endl;
+         }
+      }
+      if (ext_setsockopt(gMPTCPSocket, IPPROTO_TCP, TCP_MULTIPATH_SCHEDULER, gScheduler, strlen(gScheduler)) < 0) {
+         if(strcmp(gScheduler, "default") != 0) {
+            std::cerr << "WARNING: Failed to set TCP_MULTIPATH_SCHEDULER on socket - "
+                        << strerror(errno) << "!" << std::endl;
+         }
+      }
+      if(setBufferSizes(gMPTCPSocket, gSndBufSize, gRcvBufSize) == false) {
+         exit(1);
       }
    }
 #endif
 
    gUDPSocket = createAndBindSocket(AF_UNSPEC, SOCK_DGRAM, IPPROTO_UDP, localPort,
-                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
+                                    gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true, gBindV6Only);
    if(gUDPSocket < 0) {
       cerr << "ERROR: Failed to create and bind UDP socket - "
            << strerror(errno) << "!" << endl;
@@ -871,7 +940,7 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
 
 #ifdef HAVE_DCCP
    gDCCPSocket = createAndBindSocket(AF_UNSPEC, SOCK_DCCP, IPPROTO_DCCP, localPort,
-                                     gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
+                                     gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true, gBindV6Only);
    if(gDCCPSocket < 0) {
       cerr << "NOTE: Your kernel does not provide DCCP support." << endl;
    }
@@ -881,10 +950,13 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
                << strerror(errno) << "!" << std::endl;
       exit(1);
    }
+   if(setBufferSizes(gDCCPSocket, gSndBufSize, gRcvBufSize) == false) {
+      exit(1);
+   }
 #endif
 
    gSCTPSocket = createAndBindSocket(AF_UNSPEC, SOCK_STREAM, IPPROTO_SCTP, localPort,
-                                     gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true);
+                                     gLocalAddresses, (const sockaddr_union*)&gLocalAddressArray, true, gBindV6Only);
    if(gSCTPSocket < 0) {
       cerr << "ERROR: Failed to create and bind SCTP socket - "
            << strerror(errno) << "!" << endl;
@@ -907,6 +979,9 @@ void passiveMode(int argc, char** argv, const uint16_t localPort)
            << strerror(errno) << "!" << endl;
       exit(1);
    }
+   if(setBufferSizes(gSCTPSocket, gSndBufSize, gRcvBufSize) == false) {
+      exit(1);
+   }
 
 
    // ====== Print status ===================================================
@@ -1016,12 +1091,22 @@ void activeMode(int argc, char** argv)
    const char*      configName        = "";
    uint8_t          protocol          = 0;
    Flow*            lastFlow          = NULL;
+   
+   // ------ Handle global parameters first ---------------------------------
    for(int i = 2;i < argc;i++) {
+      if(handleGlobalParameter(argv[i])) {
+         argv[i] = NULL;
+      }
+   }
+   
+   // ------ Handle other parameters ----------------------------------------
+   for(int i = 2;i < argc;i++) {
+      if(argv[i] == NULL) {
+         continue;   // Parameter has already been handled above!
+      }
       if(argv[i][0] == '-') {
          lastFlow = NULL;
-         if(handleGlobalParameter(argv[i])) {
-         }
-         else if(strcmp(argv[i], "-tcp") == 0) {
+         if(strcmp(argv[i], "-tcp") == 0) {
             protocol = IPPROTO_TCP;
          }
          else if(strcmp(argv[i], "-udp") == 0) {
