@@ -1,7 +1,7 @@
-/* $Id: netperfmeterpackets.h 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -71,6 +71,7 @@ struct NetPerfMeterAcknowledgeMessage
 
 #define NETPERFMETER_PATHMGR_LENGTH       16
 #define NETPERFMETER_CC_LENGTH            16
+#define NETPERFMETER_SCHEDULER_LENGTH     16
 
 struct NetPerfMeterOnOffEvent
 {
@@ -113,6 +114,7 @@ struct NetPerfMeterAddFlowMessage
    uint16_t               NDiffPorts;
    char                   PathMgr[NETPERFMETER_PATHMGR_LENGTH];
    char                   CongestionControl[NETPERFMETER_CC_LENGTH];
+   char                   Scheduler[NETPERFMETER_SCHEDULER_LENGTH];
 
    uint16_t               OnOffEvents;
    NetPerfMeterOnOffEvent OnOffEvent[];
