@@ -1,7 +1,7 @@
-/* $Id: tools.cc 1933 2014-11-17 07:41:37Z dreibh $
+/* $Id$
  *
  * Network Performance Meter
- * Copyright (C) 2009-2015 by Thomas Dreibholz
+ * Copyright (C) 2009-2016 by Thomas Dreibholz
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -229,13 +229,10 @@ size_t getSocklen(const struct sockaddr* address)
    switch(address->sa_family) {
       case AF_INET:
          return(sizeof(struct sockaddr_in));
-       break;
       case AF_INET6:
          return(sizeof(struct sockaddr_in6));
-       break;
       default:
          return(sizeof(struct sockaddr));
-       break;
    }
 }
 
@@ -316,7 +313,7 @@ bool address2string(const struct sockaddr* address,
             snprintf(buffer, length, "%s", inet_ntoa(ipv4address->sin_addr));
          }
          return(true);
-       break;
+
       case AF_INET6:
          ipv6address = (const struct sockaddr_in6*)address;
          if( (!hideScope) &&
@@ -344,10 +341,10 @@ bool address2string(const struct sockaddr* address,
             return(true);
          }
        break;
+
       case AF_UNSPEC:
          safestrcpy(buffer, "(unspecified)", length);
          return(true);
-       break;
    }
    return(false);
 }
@@ -471,9 +468,9 @@ bool string2address(const char*           string,
          ipv6address->sin6_len  = sizeof(struct sockaddr_in6);
 #endif
        break;
+
       default:
          return(false);
-       break;
    }
 
    freeaddrinfo(res);
@@ -532,13 +529,10 @@ uint16_t getPort(const struct sockaddr* address)
       switch(address->sa_family) {
          case AF_INET:
             return(ntohs(((struct sockaddr_in*)address)->sin_port));
-          break;
          case AF_INET6:
             return(ntohs(((struct sockaddr_in6*)address)->sin6_port));
-          break;
          default:
             return(0);
-          break;
       }
    }
    return(0);
@@ -553,11 +547,9 @@ bool setPort(struct sockaddr* address, uint16_t port)
          case AF_INET:
             ((struct sockaddr_in*)address)->sin_port = htons(port);
             return(true);
-          break;
          case AF_INET6:
             ((struct sockaddr_in6*)address)->sin6_port = htons(port);
             return(true);
-          break;
       }
    }
    return(false);
@@ -571,7 +563,8 @@ int createAndBindSocket(const int             family,
                         const uint16_t        localPort,
                         const unsigned int    localAddresses,
                         const sockaddr_union* localAddressArray,
-                        const bool            listenMode)
+                        const bool            listenMode,
+                        const bool            bindV6Only)
 {
    unsigned int localAddressCount = localAddresses;
 
@@ -615,12 +608,17 @@ int createAndBindSocket(const int             family,
    if(sd < 0) {
       return(-2);
    }
-#ifdef IPV6_BINDV6ONLY
+#ifdef IPV6_V6ONLY
    if(socketFamily == AF_INET6) {
       // Accept IPv4 and IPv6 connections.
-      int on = 0;
-      ext_setsockopt(sd, IPPROTO_IPV6, IPV6_BINDV6ONLY, (char *)&on, sizeof(on));
+      const int on = (bindV6Only == true) ? 1 : 0;
+      // printf("IPV6_V6ONLY=%d\n", on);
+      if(ext_setsockopt(sd, IPPROTO_IPV6, IPV6_V6ONLY, &on, sizeof(on)) < 0) {
+         return(-2);
+      }
    }
+#else
+#error IPV6_V6ONLY not defined?! Please create a bug report and provide some information about your OS!
 #endif
 // FIXME! Add proper, platform-independent code here!
 #ifndef __linux__
@@ -629,8 +627,10 @@ int createAndBindSocket(const int             family,
    if((protocol == IPPROTO_MPTCP) || (protocol == IPPROTO_TCP)) {
       const int cmtOnOff = (protocol == IPPROTO_MPTCP);
       if(ext_setsockopt(sd, IPPROTO_TCP, TCP_MULTIPATH_ENABLE, &cmtOnOff, sizeof(cmtOnOff)) < 0) {
-         if(protocol == IPPROTO_MPTCP) {
-            return(-2);
+         if(ext_setsockopt(sd, IPPROTO_TCP, TCP_MULTIPATH_ENABLE_LEGACY, &cmtOnOff, sizeof(cmtOnOff)) < 0) {   // FIXME: get rid of legacy option number!
+            if(protocol == IPPROTO_MPTCP) {
+               return(-2);
+            }
          }
       }
    }
@@ -1020,16 +1020,12 @@ const char* getRandomGeneratorName(const uint8_t rng)
    switch(rng) {
       case RANDOM_CONSTANT:
          return("constant");
-       break;
       case RANDOM_EXPONENTIAL:
          return("exponential");
-       break;
       case RANDOM_UNIFORM:
          return("uniform");
-       break;
       case RANDOM_PARETO:
          return("pareto");
-       break;
    }
    return("(invalid!)");
 }
@@ -1088,9 +1084,6 @@ uint32_t random32()
             return(number);
          }
          RandomSource = RS_CLIB;
-      case RS_CLIB:
-         return(random());
-       break;
       case RS_TRY_DEVICE:
          RandomDevice = fopen("/dev/urandom", "r");
          if(RandomDevice != NULL) {
@@ -1103,9 +1096,11 @@ uint32_t random32()
          }
          RandomSource = RS_CLIB;
          srandom((unsigned int)(getMicroTime() & (uint64_t)0xffffffff));
-      break;
+       break;
    }
-   return(random());
+   const uint16_t a = random() & 0xffff;
+   const uint16_t b = random() & 0xffff;
+   return( (((uint32_t)a) << 16) | (uint32_t)b );
 #endif
 }
 
@@ -1125,20 +1120,39 @@ double randomExpDouble(const double p)
 
 
 /* ###### Get pareto-distributed double random value ##################### */
-// m = the minimum value
-// k = the k value 
-double randomParetoDouble(const double m, const double k)
+// Parameters:
+// location = the location parameter (also: scale parameter): x_m, x_min or m
+// shape    = the shape parameter: alpha or k
+//
+// Based on rpareto from GNU R's VGAM package
+// (http://cran.r-project.org/web/packages/VGAM/index.html):
+// rpareto <- function (n, location, shape) 
+// {
+//     ans <- location/runif(n)^(1/shape)
+//     ans[location <= 0] <- NaN
+//     ans[shape <= 0] <- NaN
+//     ans
+// }
+//
+// Some description:
+// http://en.wikipedia.org/wiki/Pareto_distribution
+//
+// Mean: E(X) = shape*location / (shape - 1) for alpha > 1
+// => location = E(X)*(shape - 1) / shape
+//
+double randomParetoDouble(const double location, const double shape)
 {
-   assert(k > 0.0);
+   assert(shape > 0.0);
 
    double r = randomDouble();
    while ((r <= 0.0) || (r >= 1.0)) {
       r = randomDouble();
    }
-   return( m * pow(r, -1.0 / k) );
+   return( location / pow(r, 1.0 / shape) );
 }
 
 
+/* ###### poll() to select() wrapper to work around broken Apple poll() ## */
 #ifdef __APPLE__
 #warning Using poll() to select() wrapper to work around broken Apple poll().
 int ext_poll_wrapper(struct pollfd* fdlist, long unsigned int count, int time)
@@ -1208,6 +1222,7 @@ int ext_poll_wrapper(struct pollfd* fdlist, long unsigned int count, int time)
 #endif
 
 
+/* ###### Word-around for lksctp bug ##################################### */
 ssize_t sctp_send_fixed(int                           sd,
                         const void*                   data,
                         size_t                        len,
@@ -1235,3 +1250,52 @@ ssize_t sctp_send_fixed(int                           sd,
    memcpy(sri, sinfo, sizeof(struct sctp_sndrcvinfo));
    return(ext_sendmsg(sd, &msg, msg.msg_flags));
 }
+
+
+/* ###### Configure send and receive buffer sizes ######################## */
+bool setBufferSizes(int sd, const int sndBufSize, const int rcvBufSize)
+{
+   if(sndBufSize > 0) {
+      if(ext_setsockopt(sd, SOL_SOCKET, SO_SNDBUF, &sndBufSize, sizeof(sndBufSize)) < 0) {
+         std::cerr << "ERROR: Failed to configure send buffer size (SO_SNDBUF option) - "
+                   << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+      int newBufferSize = 0;
+      socklen_t newBufferSizeLength = sizeof(newBufferSize);
+      if(ext_getsockopt(sd, SOL_SOCKET, SO_SNDBUF, &newBufferSize, &newBufferSizeLength) < 0) {
+         std::cerr << "ERROR: Failed to obtain receive send size (SO_SNDBUF option) - "
+                   << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+      // printf("SET-SNDBUF: sd=%d - %d (requested %d)\n", sd, newBufferSize, sndBufSize);
+      if(newBufferSize < sndBufSize) {
+         std::cerr << "ERROR: actual send buffer size < configured send buffer size: "
+                   << newBufferSize << " < " << sndBufSize
+                   << std::endl;
+         return(false);
+      }
+   }
+   if(rcvBufSize > 0) {
+      if(ext_setsockopt(sd, SOL_SOCKET, SO_RCVBUF, &rcvBufSize, sizeof(rcvBufSize)) < 0) {
+         std::cerr << "ERROR: Failed to configure receive buffer size (SO_RCVBUF option) - "
+                   << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+      int newBufferSize = 0;
+      socklen_t newBufferSizeLength = sizeof(newBufferSize);
+      if(ext_getsockopt(sd, SOL_SOCKET, SO_RCVBUF, &newBufferSize, &newBufferSizeLength) < 0) {
+         std::cerr << "ERROR: Failed to obtain receive buffer size (SO_RCVBUF option) - "
+                   << strerror(errno) << "!" << std::endl;
+         return(false);
+      }
+      // printf("SET-RCVBUF: sd=%d - %d (requested %d)\n", sd, newBufferSize, rcvBufSize);
+      if(newBufferSize < rcvBufSize) {
+         std::cerr << "ERROR: actual receive buffer size < configured receive buffer size: "
+                   << newBufferSize << " < " << rcvBufSize
+                   << std::endl;
+         return(false);
+      }
+   }
+   return(true);
+}
